@@ -55,7 +55,7 @@ sub new {
     );
 
     my $combodata = Gtk2::Ex::DbLinker::RdbDataManager->new(
-        data => Rdb::Langue::Manager->get_langues( order_by => 'langue' ),
+        data => Rdb::Langue::Manager->get_langues( sort_by => 'langue' ),
         meta => Rdb::Langue->meta,
     );
 
