@@ -20,35 +20,21 @@ use constant {
     STATUS_COLUMN => 0
 };
 
-my %fieldtype = (
-    serial  => "number",
-    varchar => "text",
-    char    => "text",
-    integer => "number",
-    enum    => "text",
-    date    => "time",
-    boolean => "boolean",
-    set     => "text",
-);
-
 #if number is build with GLib::Int there a default 0 value that is added in a new row and that cannot be reset to undef
 #auto incremented primary keys are not correct then.
 #Putting Glib::String prevents this
 
 my %render = (
-    text   => [ sub { return Gtk2::CellRendererText->new; }, "Glib::String", ],
-    hidden => [ sub { return Gtk2::CellRendererText->new; }, "Glib::String" ],
-    number => [ sub { return Gtk2::CellRendererText->new; }, "Glib::String" ],
-    toggle =>
-      [ sub { return Gtk2::CellRendererToggle->new; }, "Glib::Boolean" ],
-    combo => [ sub { return Gtk2::CellRendererCombo->new; }, "Glib::String" ],
-    progress =>
-      [ sub { return Gtk2::CellRendererProgress->new; }, "Glib::Int" ],
-    status_column =>
-      [ sub { return Gtk2::CellRendererPixbuf->new; }, "Glib::Int" ],
-    image => [ sub { return Gtk2::CellRendererPixbuf->new; }, "Glib::Int" ],
-    boolean => [ sub { return Gtk2::CellRendererText->new; }, "Glib::String" ],
-    time    => [ sub { return Gtk2::CellRendererText->new; }, "Glib::String" ],
+    text          => [ sub { return Gtk2::CellRendererText->new; },     "Glib::String", ],
+    hidden        => [ sub { return Gtk2::CellRendererText->new; },     "Glib::String" ],
+    number        => [ sub { return Gtk2::CellRendererText->new; },     "Glib::String" ],
+    toggle        => [ sub { return Gtk2::CellRendererToggle->new; },   "Glib::Boolean" ],
+    combo         => [ sub { return Gtk2::CellRendererCombo->new; },    "Glib::String" ],
+    progress      => [ sub { return Gtk2::CellRendererProgress->new; }, "Glib::Int" ],
+    status_column => [ sub { return Gtk2::CellRendererPixbuf->new; },   "Glib::Int" ],
+    image         => [ sub { return Gtk2::CellRendererPixbuf->new; },   "Glib::Int" ],
+    boolean       => [ sub { return Gtk2::CellRendererText->new; },     "Glib::String" ],
+    time          => [ sub { return Gtk2::CellRendererText->new; },     "Glib::String" ],
 );
 
 #
@@ -60,10 +46,9 @@ my %render = (
 #all this to have $self as the first arg in the xxx_edited sub ...
 #
 my %signals = (
-    'Gtk2::CellRendererText' => [ 'edited', sub { shift->_cell_edited(@_) } ],
-    'Gtk2::CellRendererToggle' =>
-      [ 'toggled', sub { shift->_toggle_edited(@_) } ],
-    'Gtk2::CellRendererCombo' => [ 'edited', sub { shift->_combo_edited(@_) } ],
+    'Gtk2::CellRendererText'   => [ 'edited',  sub { shift->_cell_edited(@_) } ],
+    'Gtk2::CellRendererToggle' => [ 'toggled', sub { shift->_toggle_edited(@_) } ],
+    'Gtk2::CellRendererCombo'  => [ 'edited',  sub { shift->_combo_edited(@_) } ],
 
 );
 
@@ -102,85 +87,24 @@ sub new {
     $self->{hcols} = \%hcols;
     $self->{log}->debug( "cols: " . join( " ", @cols ) );
 
-    $self->_setup_fields;
+    $self->{ds_helper} = Gtk2::Ex::DbLinker::DatasheetHelper->new(
+        cols       => $self->{cols},
+        dman       => $self->{dman},
+	#col_number => sub { $self->colnumber_from_name( $_[0] ) },
+    );
+    ( $self->{fields}, undef) = 
+        $self->{ds_helper}->setup_fields(
+	    allfields=> $self->{fields}, 
+	    cols=> $self->{cols}, 
+	    status_col => {name => "_status_column_", renderer      => "status_column", header_markup => ""}
+            );
 
     $self->_setup_treeview;
-    $self->{ds_helper} = Gtk2::Ex::DbLinker::DatasheetHelper->new(
-	   cols => $self->{cols},
-	    dman => $self->{dman},
-	    col_number => sub {$self->colnumber_from_name($_[0])},
-    );
+
     return $self;
 
 }
 
-sub _setup_fields {
-    my ($self) = @_;
-
-# If there are no field definitions, then create some from our fieldlist from the database
-    if ( !$self->{fields} ) {
-        my $no_of_fields = scalar @{ $self->{cols} };
-        my $field_percentage =
-          $no_of_fields < 8
-          ? 100 / $no_of_fields
-          : 12.5
-          ;    # Don't set percentages < 12.5 - doesn't really work so well ...
-        for my $field ( @{ $self->{cols} } ) {
-            my $gtktype = $fieldtype{ $self->{dman}->get_field_type($field) };
-            push @{ $self->{fields} },
-              {
-                name      => $field,
-                x_percent => $field_percentage,
-                renderer  => $gtktype,
-              };
-            $self->{log}
-              ->debug( " * set field : " . $field . " renderer : " . $gtktype );
-        }
-    }
-
-# Put a _status_column_ at the front of $self->{fieldlist} and also $self->{fields}
-# so we don't have off-by-one BS everywhere
-# unshift @{$self->{cols}}, "_status_column_";
-
-    unshift @{ $self->{fields} },
-      {
-        name          => "_status_column_",
-        renderer      => "status_column",
-        header_markup => ""
-      };
-
-    my $column_no;
-    for my $field ( @{ $self->{fields} } ) {
-
-        # $self->{log}->debug("field name : " . $field->{name});
-
-        $self->{colname_to_number}->{ $field->{name} } = $column_no;
-
-        if ( !$field->{renderer} ) {
-
-#my $x = ( $fieldtype{ $self->{fieldsType}->{$field->{name}}}  ?  $fieldtype{$self->{fieldsType}->{$field->{name}}} : "text");
-            my $ftype = $self->{dman}->get_field_type( $field->{name} );
-            my $x = ( $fieldtype{$ftype} ? $fieldtype{$ftype} : "text" );
-            $field->{renderer} = $x;
-            $self->{log}->debug(
-                "reset renderer for field " . $field->{name} . " to " . $x );
-
-        }
-
-        #if ($field->{renderer} eq "combo") {
-        #    $self->setup_combo($field->{name});
-        #}
-        $self->{log}->debug( " ** set field : "
-              . $field->{name}
-              . " renderer : "
-              . $field->{renderer} );
-        $field->{column} = $column_no++;
-    }
-
-    #setupfields_end
-    #
-}    # setup_fields
-
 sub _setup_treeview {
     my ($self) = @_;
 
@@ -217,7 +141,7 @@ sub _setup_treeview {
             }
         }
 
-# Icons don't seem to take up the entire cell, so we need some more room. This will do ...
+        # Icons don't seem to take up the entire cell, so we need some more room. This will do ...
         $self->{status_icon_width} += 10;
     }
 
@@ -229,25 +153,14 @@ sub _setup_treeview {
 
         my $cell_ref = ref $renderer;
 
-        $self->{log}->debug( "Setup tv : field name : "
-              . $field->{name} . " "
-              . $field->{column}
-              . " ref: "
-              . $cell_ref );
+        $self->{log}->debug( "Setup tv : field name : " . $field->{name} . " " . $field->{column} . " ref: " . $cell_ref );
 
-        push @{ $self->{ $treeview_type . "_treestore_def" } },
-          $render{ $field->{renderer} }[1];
+        push @{ $self->{ $treeview_type . "_treestore_def" } }, $render{ $field->{renderer} }[1];
 
         if ( $field->{renderer} eq "status_column" ) {
 
-            #	$renderer = Gtk2::CellRendererPixbuf->new;
-            #
-            #$renderer = $render{$field->{renderer}}[0]();
-
-            $field->{ $treeview_type . "_column" } =
-              Gtk2::TreeViewColumn->new_with_attributes( "", $renderer );
-            $self->{$treeview_type}
-              ->append_column( $field->{ $treeview_type . "_column" } );
+            $field->{ $treeview_type . "_column" } = Gtk2::TreeViewColumn->new_with_attributes( "", $renderer );
+            $self->{$treeview_type}->append_column( $field->{ $treeview_type . "_column" } );
 
             # Otherwise set fixed width
             $field->{x_absolute} = $self->{status_icon_width};
@@ -263,19 +176,15 @@ sub _setup_treeview {
 
         } else {
 
-            #$renderer = Gtk2::CellRendererText->new;
-            #$renderer = $render{$field->{renderer}}();
-            # no de la col
+           # no de la col
             $renderer->{column} = $field->{column};
-	    
+
             if ( $field->{renderer} eq "toggle" ) {
 
                 $renderer->set( activatable => TRUE );
 
                 # $renderer->set( editable => TRUE );
-                $field->{ $treeview_type . "_column" } =
-                  Gtk2::TreeViewColumn->new_with_attributes( $field->{name},
-                    $renderer, 'active' => $field->{column} );
+                $field->{ $treeview_type . "_column" } = Gtk2::TreeViewColumn->new_with_attributes( $field->{name}, $renderer, 'active' => $field->{column} );
             } elsif ( $field->{renderer} eq "combo" ) {
                 $renderer->set_fixed_size( $field->{x_percent} * 20, -1 )
                   if ( $field->{x_percent} );
@@ -290,17 +199,15 @@ sub _setup_treeview {
 
                 push @combodata, "Glib::String";
 
-                $self->{log}->debug(
-                    "field name with combo renderer: " . $field->{name} );
+                $self->{log}->debug( "field name with combo renderer: " . $field->{name} );
 
                 #my $fieldtype = $self->{fieldsType}->{$field->{name}};
-                my $fieldtype =
-                  $fieldtype{ $self->{dman}->get_field_type( $field->{name} ) };
+                my $fieldtype = $self->{ds_helper}->get_gui_type($self->{dman}->get_field_type( $field->{name} ));
+		#$fieldtype{ $self->{dman}->get_field_type( $field->{name} ) };
 
                 # $self->{log}->debug("combo field type : " . $fieldtype);
-                if ( $fieldtype eq "number" )
-                {    # serial, intege but not boolean ...
-                        # $renderer->{data_type} = "numeric";
+                if ( $fieldtype eq "number" ) {    # serial, intege but not boolean ...
+                                                   # $renderer->{data_type} = "numeric";
                     $renderer->{comp} = sub {
                         my ( $a, $b, $c ) = @_;
                         return ( $c ? ( $a == $b ) : ( $a != $b ) );
@@ -313,13 +220,10 @@ sub _setup_treeview {
                         return ( $c ? ( $a eq $b ) : ( $a ne $b ) );
                     };
                 }
-                $field->{ $treeview_type . "_column" } =
-                  Gtk2::TreeViewColumn->new_with_attributes( $field->{name},
-                    $renderer, 'text' => $renderer->{col_data} );
+                $field->{ $treeview_type . "_column" } = Gtk2::TreeViewColumn->new_with_attributes( $field->{name}, $renderer, 'text' => $renderer->{col_data} );
 
             } else {
-                $self->{log}
-                  ->debug( "field name with txt renderer: " . $field->{name} );
+                $self->{log}->debug( "field name with txt renderer: " . $field->{name} );
 
                 #if ($field->{name} ~~ @apk) {
                 if ( grep /^$field->{name}$/, @apk ) {
@@ -328,9 +232,7 @@ sub _setup_treeview {
                 } else {
                     $renderer->set( editable => TRUE );
                 }
-                $field->{ $treeview_type . "_column" } =
-                  Gtk2::TreeViewColumn->new_with_attributes( $field->{name},
-                    $renderer, 'text' => $field->{column} );
+                $field->{ $treeview_type . "_column" } = Gtk2::TreeViewColumn->new_with_attributes( $field->{name}, $renderer, 'text' => $field->{column} );
 
             }
 
@@ -340,30 +242,25 @@ sub _setup_treeview {
                 $field->{ $treeview_type . "_column" }->set_visible(FALSE);
             } else {
 
-           #$renderer->signal_connect (edited => sub { $self->cell_edited(@_)});
+                #$renderer->signal_connect (edited => sub { $self->cell_edited(@_)});
                 if ( exists $signals{$cell_ref} ) {
-                    $self->{log}
-                      ->debug( " signal : " . $signals{$cell_ref}[0] );
+                    $self->{log}->debug( " signal : " . $signals{$cell_ref}[0] );
                     my $coderef = $signals{$cell_ref}[1];
 
-      # $renderer->signal_connect ( $signals{$cell_ref}[0] => $coderef, $self );
-                    $renderer->signal_connect(
-                        $signals{$cell_ref}[0] => sub { $self->$coderef(@_) } );
+                    # $renderer->signal_connect ( $signals{$cell_ref}[0] => $coderef, $self );
+                    $renderer->signal_connect( $signals{$cell_ref}[0] => sub { $self->$coderef(@_) } );
                 }
 
             }
 
-            $self->{$treeview_type}
-              ->append_column( $field->{ $treeview_type . "_column" } );
+            $self->{$treeview_type}->append_column( $field->{ $treeview_type . "_column" } );
 
             $field->{ $treeview_type . "_column" }->{renderer} = $renderer;
 
             if ( exists $field->{custom_render_functions} ) {
 
                 # $self->{suppress_gtk2_main_iteration_in_query} = TRUE;
-                $field->{ $treeview_type . "_column" }
-                  ->{custom_render_functions} =
-                  $field->{custom_render_functions};
+                $field->{ $treeview_type . "_column" }->{custom_render_functions} = $field->{custom_render_functions};
             }
 
         }    #<> status_col
@@ -382,23 +279,15 @@ sub _setup_treeview {
 
         $field->{ $treeview_type . "_column" }->set_widget($label);
 
-        if (
-            exists $field->{ $treeview_type . "_column" }
-            ->{custom_render_functions} )
-        {
+        if ( exists $field->{ $treeview_type . "_column" }->{custom_render_functions} ) {
             $field->{ $treeview_type . "_column" }->set_cell_data_func(
                 $renderer,
                 sub {
-                    my ( $tree_column, $renderer, $model, $iter,
-                        @all_other_stuff )
-                      = @_;
+                    my ( $tree_column, $renderer, $model, $iter, @all_other_stuff ) = @_;
                     $tree_column->{render_value} =
                       $model->get( $iter, $renderer->{column} );
-                    foreach my $render_function (
-                        @{ $tree_column->{custom_render_functions} } )
-                    {
-                        &$render_function( $tree_column, $renderer, $model,
-                            $iter, @all_other_stuff );
+                    foreach my $render_function ( @{ $tree_column->{custom_render_functions} } ) {
+                        &$render_function( $tree_column, $renderer, $model, $iter, @all_other_stuff );
                     }
                     return FALSE;
                 }
@@ -431,84 +320,27 @@ sub _setup_treeview {
 #the first field links the data from the table with a value in the list
 #the remaining fields are displayed in the combo
 # the type of the first field is in ->{fieldsType} the other(s) are supposed to be strings
+
 sub _setup_combo {
     my ( $self, $fieldname ) = @_;
 
-    my $column_no = $self->{colname_to_number}->{$fieldname};
-    $self->{log}->debug( "setup_combo field name : "
-          . $fieldname
-          . " col number :"
-          . $column_no );
+    my $fields = $self->{fields}[ $self->colnumber_from_name($fieldname) ];
 
-    #my @combo = @{$self->{fields}[$column_no]->{data}};
-    my $dman = $self->{fields}[$column_no]->{data_manager};
-    my $last = $dman->row_count;
+  my $col_ref =  $self->{ds_helper}->init_combo_setup(name => $fieldname, fields => $fields);
 
-    # return unless(@combo);
+   my @liste_def = $self->{ds_helper}->get_liste_def(
+	   fields =>$fields , 
+	   renderer=> \%render, 
+	   default_renderer=>"Glib::String",
+	   col_ref => $col_ref,
+   );
+    $self->{ds_helper}->setup_combo(
+        fields  => $fields,
+        name    => $fieldname,
+        col_ref => $col_ref,
+        model   => Gtk2::ListStore->new(@liste_def)
 
-    my @liste_def;
-
-    # my $firstrow = $combo[0];
-
-    my @cols;    # = $firstrow->meta->column_names;
-    if ( $self->{fields}[$column_no]->{fieldnames} ) {
-        @cols = @{ $self->{fields}[$column_no]->{fieldnames} };
-    } else {
-        @cols = $dman->get_field_names;
-    }
-
-    $self->{log}->debug( "setup_combo cols : " . join( " ", @cols ) );
-
-    # my $rdbtype = $fieldtype{ $self->{fieldsType}->{ $cols[0]  } };
-    #  my $rdbtype = $fieldtype{ $self->{dman}->get_field_type ($cols[0] ) };
-    my $rdbtype    = $fieldtype{ $dman->get_field_type( $cols[0] ) };
-    my $first_type = $render{$rdbtype}[1];
-    my $pos        = 0;
-    foreach my $name (@cols) {
-        if ( $pos++ == 0 ) {
-
-            push @liste_def, $render{$rdbtype}[1];
-
-            # push @liste_def, "Glib::String";
-        } else {
-            push @liste_def, "Glib::String";
-        }
-
-    }
-
-    my $model = Gtk2::ListStore->new(@liste_def);
-
-    #foreach  my $row ( @combo ) {
-    for ( my $row_pos = 0 ; $row_pos < $last ; $row_pos++ ) {
-
-        my @model_row;
-
-        # $self->{log}->debug("Datasheet setupcombo set row pos " . $row_pos);
-        $dman->set_row_pos($row_pos);
-
-        push @model_row, $model->append;
-        $pos = 0;
-        foreach my $name (@cols) {
-
-            #push @model_row, $pos++, $row->$name();
-            push @model_row, $pos++, $dman->get_field($name);
-
-# $self->{log}->debug("field: " . $name . " val : ". $row->$name() . " pos: ". $pos);
-#$pos++;
-        }
-
-        # print Dumper(@model_row);
-        $model->set(@model_row);
-
-        # $model->set_text_column(1);
-
-    }    #for
-         # @cols = $self->{treeview}->get_columns;
-         # $self->{log}->debug("last col is : " . scalar @cols);
-         # my $column = $self->{treeview}->get_column($column_no);
-         #  my $renderer = ($column->get_cell_renderers)[0];
-
-    return $model;
+    );
 
 }
 
@@ -543,51 +375,34 @@ sub update {
                 my $record_status = UNCHANGED;
 
                 # $self->{log}->debug("Col " . $column . " added");
-                push @model_row, $liststore->append, STATUS_COLUMN,
-                  $record_status;
+                push @model_row, $liststore->append, STATUS_COLUMN, $record_status;
 
             } else {
                 my $x = "";
 
                 # if ( $field->{name} ~~ @{$self->{cols}}) {
                 if ( defined $self->{hcols}->{ $field->{name} } ) {
-
-            # $self->{log}->debug("query: " .  $field->{name} . " row: " . $i );
-
+                    # $self->{log}->debug("query: " .  $field->{name} . " row: " . $i );
                     $x = $self->{dman}->get_field( $field->{name} );
-
-# $self->{log}->debug( $field->{name} . " " . (defined $x ? "x: " . $x : "x undefined"));
+                    # $self->{log}->debug( $field->{name} . " " . (defined $x ? "x: " . $x : "x undefined"));
                     if ( defined $x ) {
-
-# $self->{log}->debug( $field->{name} . " " . ( $x ne "" ? "x: " . $x : "x zls"));
+                        # $self->{log}->debug( $field->{name} . " " . ( $x ne "" ? "x: " . $x : "x zls"));
                         $x = ( $x eq $self->{null_string} ? "" : $x );
                     }
 
                 } else {
-                    $self->{log}->debug( "update: "
-                          . $field->{name}
-                          . " not found in "
-                          . join( " ", @{ $self->{cols} } ) );
+                    $self->{log}->debug( "update: " . $field->{name} . " not found in " . join( " ", @{ $self->{cols} } ) );
                 }
-
-# $self->{log}->debug("field: ". $field->{name} . " col.: " . $column . " value: " . (defined $x?$x:" undef "));
-
+                # $self->{log}->debug("field: ". $field->{name} . " col.: " . $column . " value: " . (defined $x?$x:" undef "));
                 push @model_row, $column, $x;
-
                 #die unless defined($x);
                 if ( $field->{renderer} eq "combo" && defined $x && $x ne "" ) {
-                    my @renderers = $field->{ $treeview_type . "_column" }
-                      ->get_cell_renderers;
+                    my @renderers  = $field->{ $treeview_type . "_column" }->get_cell_renderers;
                     my $combomodel = $renderers[0]->get("model");
-
-# $self->{log}->debug("data-t: " . $field->{ $treeview_type . "_column" }->{renderer}->{data_type});
-                    my $value = $self->_combo_value( $combomodel, $x,
-                        $field->{ $treeview_type . "_column" }->{renderer}
-                          ->{comp} );
-
+                    # $self->{log}->debug("data-t: " . $field->{ $treeview_type . "_column" }->{renderer}->{data_type});
+                    my $value = $self->_combo_value( $combomodel, $x, $field->{ $treeview_type . "_column" }->{renderer}->{comp} );
                     # push @combo_values, $value;
                     push @model_row, $renderers[0]->{col_data}, $value;
-
                 }
             }    #else
             $column++;
@@ -602,25 +417,21 @@ sub update {
     }    # foreach row
 
     if ( $self->{row_select_signal} ) {
-        $self->{treeview}->get_selection->signal_handler_disconnect(
-            $self->{row_select_signal} );
+        $self->{treeview}->get_selection->signal_handler_disconnect( $self->{row_select_signal} );
     }
 
     $self->{log}->debug("update done");
+    $self->{changed} = FALSE;
 
     #  if ( $self->{on_changed} ) {
     $self->{log}->debug("binding on_changed callback");
-    $self->{changed_signal} =
-      $liststore->signal_connect( "row-changed" => sub { $self->_changed(@_); }
-      );
+    $self->{changed_signal} = $liststore->signal_connect( "row-changed" => sub { $self->_changed(@_); } );
 
     # }
 
     if ( $self->{on_row_select} ) {
         $self->{log}->debug("binding row_select callback");
-        $self->{row_select_signal} =
-          $self->{treeview}->get_selection->signal_connect(
-            changed => sub { $self->{on_row_select}(@_); } );
+        $self->{row_select_signal} = $self->{treeview}->get_selection->signal_connect( changed => sub { $self->{on_row_select}(@_); } );
     }
 
     $self->{treeview}->set_model($liststore);
@@ -630,8 +441,8 @@ sub update {
 
 sub get_column_value {
 
-# This function returns the value in the requested column in the currently selected row
-# If multi_select is turned on and more than 1 row is selected, it looks in the 1st row
+    # returns the value in the requested column in the currently selected row
+    # If multi_select is turned on and more than 1 row is selected, it looks in the 1st row
 
     my ( $self, $sql_fieldname ) = @_;
 
@@ -649,28 +460,10 @@ sub get_column_value {
         my $column_no = $self->colnumber_from_name($sql_fieldname);
         my $value = $model->get( $model->get_iter($selected_path), $column_no );
 
-=for comment
-        # Strip out dollars and commas for numeric columns
-        # We *don't* look for a number column with currency turned on,
-        # because sometimes you don't want to display currency formatting,
-        # and in this case, we still want to strip out currency formatting
-        # if people have entered it into a cell
-        
-        if ( exists $self->{fields}[$column_no]->{number} && $self->{fields}[$column_no]->{number} ) {
-            $value =~ s/[\$\,\%]//g;
-            if ( exists $self->{fields}[$column_no]->{number}->{percentage} && $self->{fields}[$column_no]->{number}->{percentage} ) {
-            	$value *= 100;
-            }
-        }
-=cut
-
         push @selected_values, $value;
 
     }
 
-# Previous behaviour was to only return the 1st selected value
-# To preserve backwards compatibility, we return a scalar if multi_select is off,
-# and we return an array if multi_select is turned on
     if ( $self->{multi_select} ) {
         return @selected_values;
     } else {
@@ -681,14 +474,12 @@ sub get_column_value {
 
 sub set_column_value {
 
-# This function sets the value in the requested column in the currently selected row
+    # sets the value in the requested column in the currently selected row
 
     my ( $self, $sql_fieldname, $value ) = @_;
 
     if ( $self->{mult_select} ) {
-        $self->{log}->debug(
-"set_column_value called with multi_select enabled -> setting value in 1st selected row"
-        );
+        $self->{log}->debug( "set_column_value called with multi_select enabled -> setting value in 1st selected row" );
     }
 
     my @selected_paths = $self->{treeview}->get_selection->get_selected_rows;
@@ -709,8 +500,9 @@ sub set_column_value {
 sub colnumber_from_name {
 
     my ( $self, $fieldname ) = @_;
-    confess ("fieldname undef") unless (defined $fieldname);
-    return $self->{colname_to_number}->{$fieldname}
+    # confess("fieldname undef") unless ( defined $fieldname );
+    # return $self->{colname_to_number}->{$fieldname}
+    return $self->{ds_helper}->colnumber_from_name($fieldname);
 
 }
 
@@ -754,146 +546,28 @@ sub _changed {
 }
 
 sub apply {
-    my $self = shift;
+    my $self  = shift;
     my $pkref = ( defined $_[0] ? $_[0] : undef );
-   my $model = $self->{treeview}->get_model;
-   my $iter    = $model->get_iter_first;
-   
-   #$self->{ds_helper}->init_iter( $model, $self->{changed_signal});
-   $self->{ds_helper}->init(
-   	sig =>  $self->{changed_signal},
-	sig_block => sub { $self->{log}->debug("sig_block"); $model->signal_handler_block($_[0])},
-   	sig_unblock =>  sub { $model->signal_handler_unblock($_[0]) },
-   	next =>  sub {$model->iter_next($_[0])},
-   	get_val => sub {$self->{log}->debug("get_val col: ", $_[1]); my $x = $model->get($_[0], $_[1]); $self->{log}->debug("get_val found ",  $x); return $x},
-   	set_val => sub {  $self->{log}->debug("set_val", $_[2]); $model->set($_[0], $_[1], $_[2])},
-   	del_row =>  sub  { $model->remove($_[0]) },
-   	has_more_row => sub { return ( defined $iter ? 1 : 0 )},
-    	iter => $iter,
-
-   );
-    #$model->signal_handler_block( $self->{changed_signal} ); 
-    $self->{ds_helper}->apply($pkref);
-    #$model->signal_handler_unblock( $self->{changed_signal} );
-    #return FALSE;
-
-}
-
-
-sub apply_old {
-
-    my $self = shift;
-    my $pkref = ( defined $_[0] ? $_[0] : undef );
-
-    #accder au lignes modifiees: parcourir toutes celles qui sont affiches
-    #et agir selon ce qu'indique la col status
-
-    $self->{changed} = FALSE;
-    $self->{log}->debug("apply");
-    my @iters_to_remove;
-
     my $model = $self->{treeview}->get_model;
+    my $iter  = $model->get_iter_first;
 
-    my $iter    = $model->get_iter_first;
+    #$self->{ds_helper}->init_iter( $model, $self->{changed_signal});
+    $self->{ds_helper}->init_apply(
+        sig       => $self->{changed_signal},
+        sig_block => sub { $self->{log}->debug("sig_block"); $model->signal_handler_block( $_[0] ) },
+        sig_unblock => sub { $model->signal_handler_unblock( $_[0] ) },
+        next        => sub { $model->iter_next( $_[0] ) },
+        get_val     => sub { $self->{log}->debug( "get_val col: ", $_[1] ); my $x = $model->get( $_[0], $_[1] ); $self->{log}->debug( "get_val found ", $x ); return $x },
+        set_val => sub { $self->{log}->debug( "set_val", $_[2] ); $model->set( $_[0], $_[1], $_[2] ) },
+        del_row => sub { $model->remove( $_[0] ) },
+        has_more_row => sub { return ( defined $iter ? 1 : 0 ) },
+        iter => $iter,
+	status_col => STATUS_COLUMN,
+
+    );
+
+    $self->{ds_helper}->apply($pkref);
     
-    
-    my $row_pos = 0;
-    my $row;
-
-    my @fields_to_save;
-    if ($pkref) {
-        my %seen;    #remove from cols the fields received in arg
-        %seen = map { $_ => $seen{$_}++ } ( @{ $self->{cols} }, @{$pkref} );
-        @fields_to_save = grep { $seen{$_} < 1 } keys %seen;
-    } else {
-        @fields_to_save = @{ $self->{cols} };
-    }
-
-    $model->signal_handler_block( $self->{changed_signal} );
-
-    #for all the rows in the datasheet
-    while ($iter) {
-
-        my $status = $model->get( $iter, STATUS_COLUMN );
-
-        # $self->{log}->debug("status : " . $status . " row pos " . $row_pos);
-
-        $self->{dman}->set_row_pos( $row_pos++ );
-
-        # Decide what to do based on status
-        if ( $status == UNCHANGED || $status == LOCKED ) {
-            $iter = $model->iter_next($iter);
-            next;
-        }
-
-        if ( $status == INSERTED ) {    # new row for the database
-            $self->{dman}->new_row;
-        }    
-
-        if ( $status == DELETED ) {
-
-#if ($row->delete ) {$self->{log}->debug("deleting current row");} else {$self->{log}->debug("Can't delete");}
-            if ( $self->{dman}->delete ) {
-                $self->{log}->debug("deleting current row");
-            } else {
-                $self->{log}->debug("Can't delete");
-            }
-
-            #$row->delete;
-            push @iters_to_remove, $iter;
-        } else {    # changed, inserted
-
-            my $count_update;
-
-            #for my $field ( @{$self->{fields}} ) {
-            for my $name (@fields_to_save) {
-
-                #my $name = $field->{name};
-
-                # if ( $field->{name} ~~ @{$self->{cols}}){
-                if ( defined $self->{hcols}->{$name} ) {
-                    my $x =
-                      $model->get( $iter, $self->{colname_to_number}->{$name} );
-                    $count_update++;
-                    $self->{log}->debug( "Field: "
-                          . $name
-                          . " row_pos "
-                          . $row_pos
-                          . " value: "
-                          . ( $x ? $x : "undef" ) );
-                    $self->{dman}->set_field( $name, $x );
-                } else {
-                    $self->{log}->debug( "apply : "
-                          . $name
-                          . " not found in "
-                          . join( " ", @{ $self->{cols} } ) );
-                }
-
-            }
-            if ($count_update) {
-                $self->{log}->debug("saving...");
-
-                #$row->save;
-                $self->{dman}->save;
-            } else {
-                $self->{log}->debug("no field updated, not saving");
-            }
-        }
-
-        #replace the unchanged icon in the col 0
-        $model->set( $iter, STATUS_COLUMN, UNCHANGED );
-
-        #$row_pos++;
-        $iter = $model->iter_next($iter);
-
-    }    #while
-
-    foreach $iter (@iters_to_remove) {
-        $model->remove($iter);
-    }
-
-    $model->signal_handler_unblock( $self->{changed_signal} );
-
     return FALSE;
 
 }
@@ -913,23 +587,20 @@ sub insert {
     if (@columns_and_values) {
         push @new_record, @columns_and_values;
     }
-    $self->{log}
-      ->debug( "new rec default values: " . join( " ", @new_record ) );
+    $self->{log}->debug( "new rec default values: " . join( " ", @new_record ) );
     $model->set(@new_record);
 
-    $self->{treeview}->set_cursor( $model->get_path($iter),
-        $self->{fields}[0]->{treeview_column}, 0 );
+    $self->{treeview}->set_cursor( $model->get_path($iter), $self->{fields}[0]->{treeview_column}, 0 );
 
- # Now scroll the scrolled window to the end
- # Using an idle timer is required because gtk needs time to add the new row ...
- #  ... if we don't use an idle timer, we end up scrolling to the 2nd-last row
+    # Now scroll the scrolled window to the end
+    # Using an idle timer is required because gtk needs time to add the new row ...
+    #  ... if we don't use an idle timer, we end up scrolling to the 2nd-last row
 
     Glib::Idle->add(
         sub {
             my $adjustment = $self->{treeview}->get_vadjustment;
             my $upper      = $adjustment->upper;
-            $adjustment->set_value( $upper - $adjustment->page_increment -
-                  $adjustment->step_increment );
+            $adjustment->set_value( $upper - $adjustment->page_increment - $adjustment->step_increment );
         }
     );
 
@@ -963,20 +634,19 @@ sub has_changed {
 }
 
 sub get_current_row {
-	my $self = shift;
-	return $self->{curr_row};
-
+    my $self = shift;
+    return $self->{curr_row};
 
 }
 
 sub _cell_edited {
     my ( $self, $cell, $path_string, $new_text ) = @_;
-    $self->{log}->debug("_cell_edited path_String : ". $path_string);
+    $self->{log}->debug( "_cell_edited path_String : " . $path_string );
     my $path  = Gtk2::TreePath->new_from_string($path_string);
     my $model = $self->{treeview}->get_model;
     my $col   = $cell->{column};
     $self->{curr_row} = $path_string;
-    my $iter  = $model->get_iter($path);
+    my $iter = $model->get_iter($path);
 
     $model->set_value( $iter, $col, $new_text );
     return FALSE;
@@ -985,7 +655,7 @@ sub _cell_edited {
 sub _toggle_edited {
     my ( $self, $renderer, $text_path, $something ) = @_;
     my $column_no = $renderer->{column};
-    $self->{log}->debug("_toggle_edited path". $text_path);
+    $self->{log}->debug( "_toggle_edited path" . $text_path );
     $self->{curr_row} = $text_path;
     my $path      = Gtk2::TreePath->new($text_path);
     my $model     = $self->{treeview}->get_model;
@@ -1005,15 +675,17 @@ sub _combo_edited {
     # return unless ($tree);
     #  treeViewModel[path][columnNumber] = newText
     my $model = $self->{treeview}->get_model;
+
     #print( "combo_edited " . $new_text . "\n" );
-$self->{log}->debug("_combo_edited path_string: ". $path_string);
+    $self->{log}->debug( "_combo_edited path_string: " . $path_string );
+
     #	$cell->get("model");
     #	$model->set ($iter, $cell->{column}, $new_text);
     my $path   = Gtk2::TreePath->new_from_string($path_string);
     my $citer  = $renderer->{combo}->get_active_iter;
     my $cmodel = $renderer->{combo}->get_model;
     $self->{curr_row} = $path_string;
-    my $value  = $cmodel->get( $citer, 0 );
+    my $value = $cmodel->get( $citer, 0 );
     print( "combo_edited value :" . $value . "\n" );
     my $iter = $model->get_iter($path);
     $model->set( $iter, $renderer->{column},   $value );
@@ -1172,7 +844,7 @@ C<tree> a Gtk2::TreeView
 
 =item *
 
-C<fields> a reference to an array of hash. Each hash has the following key / value pairs.
+C<fields> a reference to an array of hash refs. Each hash has the following key / value pairs.
 
 =over
 
