@@ -165,6 +165,9 @@ sub _init {
         ref $self->{rec_spinner}
         ? $self->{rec_spinner}
         : $self->{builder}->get_object( $self->{rec_spinner} ) );
+     if ($self->{rec_spinner}){
+  	$self->{rec_spinner}->configure(Gtk2::Adjustment->new(1.0, 1.0, 10.0, 1.0, 10.0, 0.0), 0.1, 0);
+     }
     $self->{rec_count_label} = (
         ref $self->{rec_count_label}
         ? $self->{rec_count_label}
@@ -355,7 +358,7 @@ sub add_combo {
 
             #$column ++;
         }
-        $self->{log}->debug( "row : " . join( " ", @model_row ) );
+	#$self->{log}->debug( "row : " . join( " ", @model_row ) );
         $model->set(@model_row);
     }
     $self->{log}->debug( "add_combo: " . $i . " rows added" );
